<?php

namespace Drupal\commerce_logistics\Resolver;

use Drupal\commerce_logistics\LogisticsQueryPluginManager;
use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 */
class DefaultLogisticsQueryResolver implements LogisticsQueryTypeResolverInterface {

  /**
   * The route match.
   *
   * @var \Drupal\Core\Routing\RouteMatchInterface
   */
  protected $routeMatch;

  /**
   * @var \Drupal\commerce_logistics\LogisticsQueryPluginManager
   */
  protected $logisticsQueryManager;

  /**
   * The logistics detail storage.
   *
   * @var \Drupal\commerce_shipping\ShipmentStorage
   */
  protected $commerceShipmentStorage;

  /**
   * The shipment information storage.
   *
   * @var \Drupal\commerce_logistics\Entity\Handler\ShipmentInfoStorage
   */
  protected $shipmentInfoStorage;

  /**
   * The logistics company storage.
   *
   * @var \Drupal\commerce_shipping\ShippingMethodStorage
   */
  protected $commerceShippingMethodStorage;

  /**
   * Constructs a new OrderStoreResolver object.
   *
   * @param \Drupal\Core\Routing\RouteMatchInterface $route_match
   *   The route match.
   * @param \Drupal\commerce_logistics\LogisticsQueryPluginManager $logistics_query_manager
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function __construct(RouteMatchInterface $route_match, LogisticsQueryPluginManager $logistics_query_manager, EntityTypeManagerInterface $entity_type_manager) {
    $this->routeMatch = $route_match;
    $this->logisticsQueryManager = $logistics_query_manager;
    $this->commerceShipmentStorage = $entity_type_manager->getStorage('commerce_shipment');
    $this->shipmentInfoStorage = $entity_type_manager->getStorage('shipment_info');
    $this->commerceShippingMethodStorage = $entity_type_manager->getStorage('commerce_shipping_method');
  }

  /**
   * {@inheritdoc}
   *
   * @see container
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('plugin.manager.logistics_query'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function resolve() {
    $order = $this->routeMatch->getParameter('commerce_order');
    if ($order instanceof OrderInterface) {
      /*========Start:Query the shipment according to the order========*/
      $shipmentStorage = $this->commerceShipmentStorage;
      $shipments = $shipmentStorage->loadMultipleByOrder($order);
      $shipment = reset($shipments);
      /*========================================End============================*/

      /*========Start:Query the shipment information according to the shipment========*/
      $shipment_id = $shipment->id();
      $shipment_infos = $this->shipmentInfoStorage->loadByProperties([
        'shipment_id' => $shipment_id,
      ]);
      $shipment_info = reset($shipment_infos);
      /*========================================End====================================================*/

      /*==Start:Calculate the time interval of the latest stored shipment information and the current time==*/
      $newest_node_time = 0;
      if (!empty($newest_logistics_node)) {
        $newest_node_time = $shipment_info->get('changed');
      }
      $now = time();
      $passed_time = $now - $newest_node_time;
      /*========================================End===========================================================*/

      $interval_time = \Drupal::config('commerce_logistics.shipping_info_query')
        ->get('interval_time');
      //Whether the time interval between the stored latest shipment information and the current time exceeds the configured time interval
      if ($passed_time > $interval_time) {
        //If the configured time interval is exceeded
        //Obtain the logistics company code

        $shipping_method = $shipment->getShippingMethod();
        $shipping_method_id = $shipping_method->id();
        $shipping_method_code_config = \Drupal::config('commerce_logistics.shipping_info_query');
        $company_code = $shipping_method_code_config->get("code{$shipping_method_id}");

        //Query whether there is a query plug-in for the shipping method according to the shipping method code
        $logistics_query_plugin_definitions = $this->logisticsQueryManager->getDefinitions();
        foreach ($logistics_query_plugin_definitions as $logistics_query_plugin_definition) {
          if ($company_code == $logistics_query_plugin_definition['type']) {
            return $company_code . '_logistics_query';
          }
        }
        //If there is no logistics query plug-in with the specified shipping method code, use the logistics query platform plug-in
        return 'aggregation_logistics_query';
      }
      else {
        //If the configured time interval is not exceeded
        return 'shipment_info_query';
      }
    }
    return NULL;
  }

}
