<?php

namespace Drupal\commerce_refunds\Entity;

use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Entity\EntityChangedTrait;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\user\EntityOwnerTrait;

/**
 * Provides the Refund record entity.
 *
 * @ContentEntityType(
 *   id = "commerce_refund_record",
 *   label = @Translation("Refund record"),
 *   label_collection = @Translation("Refund records"),
 *   label_singular = @Translation("refund record"),
 *   label_plural = @Translation("refund records"),
 *   label_count = @PluralTranslation(
 *     singular = "@count refund record",
 *     plural = "@count refund records",
 *   ),
 *   base_table = "commerce_refund_record",
 *   data_table = "commerce_refund_record_field_data",
 *   translatable = "TRUE",
 *   handlers = {
 *     "access" = "Drupal\commerce_refunds\Entity\Handler\RefundRecordAccess",
 *     "route_provider" = {
 *       "html" = "Drupal\Core\Entity\Routing\DefaultHtmlRouteProvider",
 *     },
 *     "form" = {
 *       "default" = "Drupal\commerce_refunds\Form\RefundRecordForm",
 *       "add" = "Drupal\commerce_refunds\Form\RefundRecordAddForm",
 *       "edit" = "Drupal\commerce_refunds\Form\RefundRecordEditForm",
 *       "delete" = "Drupal\Core\Entity\ContentEntityDeleteForm",
 *     },
 *     "storage" =
 *   "Drupal\commerce_refunds\Entity\Handler\RefundRecordStorage",
 *     "storage_schema" =
 *   "Drupal\commerce_refunds\Entity\Handler\RefundRecordStorageSchema",
 *     "list_builder" =
 *   "Drupal\commerce_refunds\Entity\Handler\RefundRecordListBuilder",
 *     "view_builder" =
 *   "Drupal\commerce_refunds\Entity\Handler\RefundRecordViewBuilder",
 *     "views_data" =
 *   "Drupal\commerce_refunds\Entity\Handler\RefundRecordViewsData",
 *     "translation" =
 *   "Drupal\commerce_refunds\Entity\Handler\RefundRecordTranslation",
 *   },
 *   admin_permission = "administer commerce_refund_record entities",
 *   entity_keys = {
 *     "id" = "commerce_refund_record_id",
 *     "label" = "title",
 *     "uuid" = "uuid",
 *     "langcode" = "langcode",
 *     "owner" = "uid",
 *     "uid" = "uid",
 *   },
 *   links = {
 *     "add-form" = "/commerce/commerce-refund-record/add",
 *     "canonical" = "/admin/commerce/commerce-refund-record/{commerce_refund_record}",
 *     "collection" = "/admin/commerce/commerce-refund-record",
 *     "delete-form" = "/admin/commerce/commerce-refund-record/{commerce_refund_record}/delete",
 *     "edit-form" = "/admin/commerce/commerce-refund-record/{commerce_refund_record}/edit",
 *   },
 * )
 */
class RefundRecord extends ContentEntityBase implements RefundRecordInterface {

  use EntityChangedTrait;

  use EntityOwnerTrait;

  public function getOrder() {
    return $this->get('order_id')->entity;
  }

  public function getState() {
    return $this->get('state')->value;
  }

  public function setState($state) {
    $current_state = $this->getState();
    if ($current_state != $state) {
      /** @var \Drupal\commerce_order\Entity\OrderInterface $order */
      $order = $this->getOrder();
      $state_text = match ($state) {
        1 => "Partially refunded",
        2 => "Refunded",
        3 => "Rejected",
        default => "Pending",
      };
      $params = [
        'state' => $state_text,
      ];
      commerce_refunds_add_order_comment($order, 'order_refund_state', $params);
    }
    $this->set('state', $state);
    $this->save();
  }

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {
    $fields = parent::baseFieldDefinitions($entity_type);

    $fields += static::ownerBaseFieldDefinitions($entity_type);

    $fields['title'] = BaseFieldDefinition::create('string')
      ->setLabel(t("Title"))
      ->setRequired(TRUE)
      ->setTranslatable(TRUE)
      ->setSetting("max_length", 255)
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'type' => 'string',
      ])
      ->setDisplayConfigurable("view", TRUE);

    $fields['body'] = BaseFieldDefinition::create('string_long')
      ->setLabel(t('Description'))
      ->setDescription(t('Please briefly explain the reasons for requesting a refund. After receiving your refund application, the merchant will process it according to your request.'))
      ->setTranslatable(TRUE)
      ->setRequired(FALSE)
      ->setSetting("max_length", 500)
      ->setDisplayOptions("form", [
        'type' => "string_textfield",
        'weight' => 1,
      ])
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'type' => 'string',
      ])
      ->setDisplayConfigurable("view", TRUE)
      ->setDisplayConfigurable("form", TRUE);

    $fields['images'] = BaseFieldDefinition::create('image')
      ->setLabel(t('Upload photos'))
      ->setDescription(t('Take photos and upload the problems you find to the merchant.'))
      ->setCardinality(BaseFieldDefinition::CARDINALITY_UNLIMITED)
      ->setSettings([
        'file_directory' => 'REFUND_RECORD_IMAGES',
        'alt_field_required' => FALSE,
        'file_extensions' => 'png jpg jpeg',
      ])
      ->setDisplayOptions('view', [
        'label' => 'above',
        'type' => 'image',
      ])
      ->setDisplayOptions('form', [
        'label' => 'above',
        'type' => 'image_image',
        'weight' => 2,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['order_id'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t("Order"))
      ->setDescription(t("Orders to be refunded."))
      ->setTranslatable(TRUE)
      ->setRequired(FALSE)
      ->setSetting('target_type', 'commerce_order')
      ->setSetting('handler', 'default')
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'type' => 'string',
      ])
      ->setDisplayConfigurable("view", TRUE);

    $fields['uid'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t("User"))
      ->setDescription(t("User applying for refund."))
      ->setTranslatable(TRUE)
      ->setRequired(FALSE)
      ->setSetting('target_type', 'user')
      ->setSetting('handler', 'default')
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'type' => 'string',
      ])
      ->setDisplayConfigurable("view", TRUE);

    $fields['state'] = BaseFieldDefinition::create('integer')
      ->setLabel(t('State'))
      ->setDescription(t('0:Pending, 1:Partially refunded, 2:Refunded, 3:Rejected'))
      ->setTranslatable(TRUE)
      ->setRequired(FALSE)
      ->setDefaultValue(0)
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'type' => 'number_integer',
      ])
      ->setDisplayConfigurable("view", TRUE);

    $fields['created'] = BaseFieldDefinition::create('created')
      ->setLabel(t("Created"))
      ->setDescription(t("The time that the entity was created."))
      ->setTranslatable(TRUE);

    $fields['changed'] = BaseFieldDefinition::create('changed')
      ->setLabel(t("Changed"))
      ->setDescription(t("The time that the entity was last edited."))
      ->setTranslatable(TRUE);

    return $fields;
  }

}
