<?php

namespace Drupal\commerce_refunds\Plugin\views\field;

use Drupal\commerce_price\Price;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Url;
use Drupal\views\Plugin\views\field\FieldPluginBase;
use Drupal\views\ResultRow;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * TODO: class docs.
 *
 * @ViewsField("commerce_refunds_refund_application")
 */
class RefundApplication extends FieldPluginBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The commerce_shipment storage handler.
   *
   * @var \Drupal\commerce_shipping\ShipmentStorage
   */
  protected $shipmentStorage;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * The commerce_refund_record storage handler.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $refundRecordStorage;

  /**
   * The commerce_payment storage handler.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $paymentStorage;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('entity_type.manager')->getStorage('commerce_shipment'),
      $container->get('current_user'),
      $container->get('entity_type.manager')
        ->getStorage('commerce_refund_record'),
      $container->get('entity_type.manager')->getStorage('commerce_payment')
    );
  }

  /**
   * Creates a RefundApplication instance.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Entity\EntityStorageInterface $shipment_storage
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   Current user.
   * @param \Drupal\Core\Entity\EntityStorageInterface $refund_record_storage
   *   The commerce_refund_record storage handler.
   * @param \Drupal\Core\Entity\EntityStorageInterface $payment_storage
   *   The commerce_payment storage handler.
   */
  public function __construct(
    array                      $configuration,
                               $plugin_id,
                               $plugin_definition,
    EntityTypeManagerInterface $entity_type_manager,
    EntityStorageInterface     $shipment_storage,
    AccountInterface           $current_user,
    EntityStorageInterface     $refund_record_storage,
    EntityStorageInterface     $payment_storage
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->entityTypeManager = $entity_type_manager;
    $this->shipmentStorage = $shipment_storage;
    $this->currentUser = $current_user;
    $this->refundRecordStorage = $refund_record_storage;
    $this->paymentStorage = $payment_storage;
  }

  /**
   * @{inheritdoc}
   */
  public function query() {
  }

  /**
   * Define the available options
   *
   * @return array
   */
  protected function defineOptions() {
    return parent::defineOptions();
  }

  /**
   * Provide the options form.
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    parent::buildOptionsForm($form, $form_state);
  }

  /**
   * @{inheritdoc}
   */
  public function render(ResultRow $values) {
    $order = $this->getEntity($values);
    $order_id = $order->id();
    //Query the shipping state of the logistics detail sheet according to the order ID
    $shipments = $this->shipmentStorage->loadByProperties([
      'order_id' => $order_id,
    ]);
    //TODO Package splitting
    $shipment = reset($shipments);
    //The last result to be returned
    $result = NULL;
    //Get refund request link or state info.
    $refund = $this->getRefundLinkOrStateText($order_id);
    $shipment = reset($shipments);
    $state = $shipment->getState()->getId();
    //When the freight status is not "on the way", the application for refund will be displayed; Otherwise, the link to the logistics details nodes in "On the way" will be displayed.
    //1:Ready, 2:On the way, 3:Sign for, 4:Issue
    switch ($state) {
      case "ready":
      case "issue shipment":
      case "canceled":
        $result = $refund;
        break;
      case "shipped":
        $result = Link::fromTextAndUrl($this->t("On the way"), Url::fromRoute('entity.commerce_order.user_view', [
          'user' => $this->currentUser->id(),
          'commerce_order' => $order_id,
        ], []))->toString();
        break;
      case "completed":
        $shipment_infos = \Drupal::entityTypeManager()
          ->getStorage('shipment_info')
          ->loadByProperties([
            'shipment_id' => $shipment->id(),
          ]);
        $shipment_info = reset($shipment_infos);
        $last_changed_time = $shipment_info->get('changed')->value;
        $refund_time = \Drupal::config('commerce_refunds.settings')
          ->get('refund_time');
        if ($refund_time < time() - $last_changed_time) {
          $result = $this->t("Completed");
        }
        else {
          $result = $refund;
        }
        break;
    }
    return $result;

  }

  public function getRefundLinkOrStateText($order_id) {
    $refunds = $this->refundRecordStorage->loadByProperties([
      'order_id' => $order_id,
    ]);

    //The result to be returned
    if (empty($refunds)) {
      //When the order does not have relevant application records, the user can apply for refund
      $user_id = \Drupal::currentUser()->id();
      return Link::fromTextAndUrl($this->t("Apply for refund"), Url::fromRoute('entity.commerce_refund_record.add_form', [], [
        'query' => [
          'order_id' => $order_id,
          'destination' => "/user/$user_id/orders"
        ],
      ]))->toString();
    }
    else {
      $refund = reset($refunds);
      $state = $refund->get('state')->value;

      $payments = $this->paymentStorage->loadByProperties([
        'order_id' => $order_id,
      ]);

      $currency_code = \Drupal::service('commerce_currency_resolver.current_currency')
        ->getCurrency();
      $payments_refunded_amount = [
        'number' => 0,
        'currency_code' => $currency_code,
      ];
      foreach ($payments as $payment) {
        $payment_price = $payment->getRefundedAmount();
        if ($payments_refunded_amount['currency_code'] != $payment_price->getCurrencyCode()) {
          $payment_price = $payment_price::convert($currency_code);
        }
        $payments_refunded_amount['number'] += $payment_price->getNumber();
      }

      return match ($state) {
        "1" => $this->t('Partially Refunded:') . $payments_refunded_amount['number'] . $payments_refunded_amount['currency_code'],
        "2" => $this->t('Fully Refunded:') . $payments_refunded_amount['number'] . $payments_refunded_amount['currency_code'],
        "3" => $this->t('Refund Rejected'),
        default => $this->t('Refund Pending'),
      };
    }
  }

}
